// Vikunja is a to-do list application to facilitate your life.
// Copyright 2018-2021 Vikunja and contributors. All rights reserved.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public Licensee as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public Licensee for more details.
//
// You should have received a copy of the GNU Affero General Public Licensee
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package user

import (
	"time"

	"code.vikunja.io/api/pkg/notifications"

	"xorm.io/xorm"
)

// RequestDeletion creates a user deletion confirm token and sends a notification to the user
func RequestDeletion(s *xorm.Session, user *User) (err error) {
	token, err := generateNewToken(s, user, TokenAccountDeletion)
	if err != nil {
		return err
	}

	return notifications.Notify(user, &AccountDeletionConfirmNotification{
		User:         user,
		ConfirmToken: token.Token,
	})
}

// ConfirmDeletion ConformDeletion checks a token and schedules the user for deletion
func ConfirmDeletion(s *xorm.Session, user *User, token string) (err error) {
	tk, err := getToken(s, token, TokenAccountDeletion)
	if err != nil {
		return err
	}

	if tk == nil {
		// TODO: return invalid token error
		return
	}

	err = removeTokens(s, user, TokenAccountDeletion)
	if err != nil {
		return err
	}

	user.DeletionScheduledAt = time.Now()
	_, err = s.Where("id = ?", user.ID).
		Cols("deletion_scheduled_at").
		Update(user)
	return nil
}
